﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DragDropWinForms
{
	public partial class Form1 : Form
	{
		[STAThread]
		static void Main()
		{
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault( false );
			Application.Run( new Form1() );
		}

		Point _LastMouseDownPos;

		public Form1()
		{
			InitializeComponent();

			_DragMeLabel.MouseDown += _DragMeLabel_MouseDown;
			_DragMeLabel.MouseMove += _DragMeLabel_MouseMove;
			_DragMeLabel.GiveFeedback += _DragMeLabel_GiveFeedback;
			_DragMeLabel.QueryContinueDrag += _DragMeLabel_QueryContinueDrag;

			_DropToMeLabel.MouseMove += _DropToMeLabel_MouseMove;
			_DropToMeLabel.DragEnter += _DropToMeLabel_DragEnter;
			_DropToMeLabel.DragLeave += _DropToMeLabel_DragLeave;
			_DropToMeLabel.DragOver += _DropToMeLabel_DragOver;
			_DropToMeLabel.DragDrop += _DropToMeLabel_DragDrop;
			_DropToMeLabel.AllowDrop = true;
		}

		#region Dragging source
		void _DragMeLabel_MouseDown( object sender, MouseEventArgs e )
		{
			Console.WriteLine( "(src)MouseDown( {0} )", e.Location );
			_LastMouseDownPos = e.Location;
		}

		void _DragMeLabel_MouseMove( object sender, MouseEventArgs e )
		{
			Console.WriteLine( "(src)MouseMove( {0} )", e.Location );

			// 左クリックで一定距離ドラッグされた場合はD&Dを実行
			if( e.Button == MouseButtons.Left )
			{
				if( 3 < Math.Abs(e.Location.X - _LastMouseDownPos.X)
					&& 3 < Math.Abs(e.Location.Y - _LastMouseDownPos.Y) )
				{
					// D&Dでドロップ先に送信するデータを準備
					IDataObject data = new DataObject( DataFormats.Text, ":)" );

					// D&Dセッションを開始。
					// ドロップ先に
					// 「_DragMeLabelからCopyまたはMoveが可能なデータを送る」
					// と通知する。
					DragDropEffects allowedEffects = DragDropEffects.Copy | DragDropEffects.Move;
					DragDropEffects result = _DragMeLabel.DoDragDrop( data, allowedEffects );

					// D&Dの結果を判定
					if( result == DragDropEffects.None )
						MessageBox.Show( "D&D failed." );
					else if( result == DragDropEffects.Move )
						_DragMeLabel.Text = "Moved.";
					else
						_DragMeLabel.Text = "Copied.";
				}
			}
		}

		void _DragMeLabel_GiveFeedback( object sender, GiveFeedbackEventArgs e )
		{
			//Console.WriteLine( "(src)GiveFeedback" );
		}

		void _DragMeLabel_QueryContinueDrag( object sender, QueryContinueDragEventArgs e )
		{
			//Console.WriteLine( "(src)QueryContinueDrag" );
		}
		#endregion

		#region Dragging destination
		void _DropToMeLabel_MouseMove( object sender, MouseEventArgs e )
		{
			Console.WriteLine( "(dest)MouseMove( {0} )", e.Location );
		}

		void _DropToMeLabel_DragEnter( object sender, DragEventArgs e )
		{
			e.Effect = DecideDropAction( e );
			Console.WriteLine( "(dest)DragEnter --> {0}", e.Effect );
		}

		void _DropToMeLabel_DragLeave( object sender, EventArgs e )
		{
			Console.WriteLine( "(dest)DragLeave" );
		}

		void _DropToMeLabel_DragOver( object sender, DragEventArgs e )
		{
			e.Effect = DecideDropAction( e );
			Console.WriteLine( "(dest)DragOver --> {0}", e.Effect );
		}

		void _DropToMeLabel_DragDrop( object sender, DragEventArgs e )
		{
			e.Effect = DecideDropAction( e );
			Console.WriteLine( "(dest)DragDrop --> {0}", e.Effect );

			switch( e.Effect )
			{
				case DragDropEffects.Copy:
					_DropToMeLabel.Text = (string)e.Data.GetData( DataFormats.Text );
					break;

				case DragDropEffects.Move:
					_DropToMeLabel.Text = (string)e.Data.GetData( DataFormats.Text );
					break;

				default:
					break;
			}
		}

		DragDropEffects DecideDropAction( DragEventArgs e )
		{
			Point pos = new Point( e.X, e.Y );
			pos = _DropToMeLabel.PointToClient( pos );

			// 単純に _DropToMeLabel の左半分ならば Copy、右半分ならば Move とする
			if( pos.X < (_DropToMeLabel.Width / 2) )
			{
				return DragDropEffects.Copy;
			}
			else
			{
				return DragDropEffects.Move;
			}
		}
		#endregion
	}
}